require 'sinatra/base'
require 'active_record'
#require 'sinatra/reloader' #if ENV['RACK_ENV'] == "development"
require 'pry'
require 'json'
require 'haml'
require 'sprockets'
require 'coffee_script'
require "sass"
require "rdiscount"
require "perkins/auth/github"
require "perkins/assets"
require "eventmachine"

module Perkins

  class Server < Sinatra::Application

    attr_reader :app
    set server: 'rainbows', connections: []
    set :sessions, true
    set :foo, 'bar'
    enable :sessions
    #set :environment, :production
    set :session_secret, '*&(^B234'

    configure :development do
      #register Sinatra::Reloader
    end

    use Perkins::Assets

    register Perkins::Auth::Github

    before /.*/ do
      if request.url.match(/.json$/)
        request.accept.unshift('application/json')
        request.path_info = request.path_info.gsub(/.json$/,'')
      end
    end

    #authenticate all -
    before do
      unless %w[stream sse logout unauthenticated /auth/github/callback /repos/receiver badge].include? request.path_info.split('/')[1]
        authenticate!
      end
    end

    #render index if not ajax request!
    before do
      return if %w[stream sse logout unauthenticated /auth/github/callback /repos/receiver badge config].include? request.path_info.split('/')[1]
      unless request.xhr?
        halt 200, haml( :index , locals: default_data)
      end
    end

    get '/' do
      haml :index , locals: default_data
    end

    get '/config' do
      settings.perkins_application.to_json
    end

    #get '/repos/sync' do
    #  @github_repos = build_data
    #end

    get '/repos' do
      content_type :json
      repos = Perkins::Repo.added.all
      repos.to_json
    end

    post '/repos/receiver' do
      #NEEDS VALIDATIONS
      payload = JSON.parse request.body.read
      repo = Perkins::Repo.added.find_by(gb_id: payload["repository"]["id"])
      #repo.load_git
      pushed_branch = payload["ref"].split('/').last
      repo.add_commit(payload["after"], pushed_branch)
      "received post #{payload}"
    end

    get '/repos/add/:id' do
      #ADD REPO HERE
      repo = github_user.api.repo(params[:id].to_i)
      Repo.sync_github_repo(repo)
      r = Repo.add_from_github(params[:id])
      r.to_json
    end

    get '/repos/:login/:name' do
      content_type :json
      repo = find_repo(params)
      #detects a recently added repo (no dir and no prev builds)
      repo.as_json(methods: [:last_report_id,  :http_url]).to_json
    end

    get '/badge' do
      content_type "image/svg+xml"
      repo = Repo.find_by(name: params[:repo])
      build = repo.build_reports.last
      request_badge({status: build.status ? "passing" : "error" , color: build.status ? "green" : "red" })
    end

    get '/repos/:login/:name/builds' do
      content_type :json
      repo = find_repo(params)
      repo.build_reports.to_json
    end

    get '/repos/:login/:name/builds/:build_id' do
      content_type :json
      repo = find_repo(params)
      record = repo.build_reports.find(params[:build_id])
      #record.to_json
      record.to_json
    end

    get '/repos/:login/:name/config' do
      content_type :json
      repo = find_repo(params)
      hook = repo.get_hook
      hook.try(:to_attrs).try(:to_json) || {}.to_json
    end

    #TODO send this through bg job
    get '/repos/:login/:name/run_commit' do
      content_type :json
      repo = find_repo(params)
      sha = github_user.api.commits(repo.name).first[:sha]
      repo.add_commit(sha, "master")
      repo.to_json
    end

    #TODO for json
    post '/repos/:login/:name/add_hook' do
      repo = find_repo(params)
      hook = repo.add_hook(params[:webhook_url])
      hook.try(:to_attrs).try(:to_json) || {}.to_json
    end

    get '/repos/:login/:name/builds/:build_id/restart' do
      content_type :json
      repo = find_repo(params)
      #repo.load_git
      record = repo.build_reports.find(params[:build_id])
      repo.add_commit(record['sha'], record['branch'])
      record.to_json
    end

    get '/me' do
      content_type :json
      a = github_user.api
      @user = a.user
      @orgs = a.orgs.map(&:login)
      @user.to_attrs.merge({orgs: @orgs}).to_json
    end

    get '/orgs/:id' do
      content_type :json
      github_organization_authenticate!(params['id'])
      org  = github_user.api.organization(params['id'])
      #orgs = github_user.api.orgs
      repos = github_user.api.org_repos(params["id"]).map(&:to_attrs)
      org.to_attrs.merge({repos: repos}).to_json
    end

    #TODO for json
    get '/myrepos' do
      repos = github_user.api.repos.map(&:to_attrs)
      github_user.api.user.to_attrs.merge({repos: repos}).to_json
    end

    @@conns = []

    get '/stream', provides: 'text/event-stream' do
      stream(:keep_open) do |out|
        @@conns << out
        out.callback { @@conns.delete(out) }
      end
    end

    Thread.new do
      redis = Redis.connect
      redis.psubscribe('message', 'message.*') do |on|
        on.pmessage do |match, channel, message|
                    
          channel = channel.sub('message.', '')

          @@conns.each do |out|
            out << "event: #{channel}\n"
            out << "data: #{message}\n\n"
          end
        end
      end
    end

    post '/sse' do
      puts "connections #{settings.connections.size}".yellow
      settings.connections.each { |out| out << "data: #{params[:msg]}\n\n" }
      204 # response without entity body
    end

    get '/logout' do
      logout!
      redirect 'https://github.com'
    end

    def initialize(args={})
      super
      #return if args.blank?
      app = Perkins::Application.instance
      
      puts "PerkinsCI #{settings.environment} environment loaded on #{app.host}:#{app.port}".green
      
      self.class.set :perkins_application, app
      self.class.set :github_options, {
        :scopes => "admin:repo_hook,repo,user:email",
        :secret => app.github_client_secret,
        :client_id => app.github_client_id,
      }
    end

    def github_repos
      @github_repos ||= persisted_repos.any? ? persisted_repos : build_data
    end

    def build_data
      Repo.sync_github_repos(github_user)
      redirect "/me"
    end

    def persisted_repos
      Repo.synced_records
    end

    def find_repo(params)
      id = "#{params[:login]}/#{params[:name]}"
      repo = Repo.find_by(name: id)
      repo
    end

    def request_badge(opts={})
      begin
        image_path = "https://img.shields.io/badge/build-#{opts[:status]}-#{opts[:color]}.svg?style=flat-square"
        url = URI.parse(image_path)
        result = Net::HTTP.get(url)
      rescue => e
        e
      end
    end

    ####################################
    #TODO: deprecate this
    def self.start(app, options)
      options = options.dup
      ENV['RACK_ENV'] = options.delete("e")

      register Sinatra::ActiveRecordExtension
      app = app.is_a?(String) ? eval(File.open(app).read) : app
      self.start_listener(app)

      configure :development do
        #register Sinatra::Reloader
        #set :database, 'sqlite:///db/development.sqlite3'
      end

      configure :production do
        #set :database, 'sqlite:///db/productions.sqlite3'
      end

      configure :test do
        #set :database, 'sqlite:///db/test.sqlite3'
      end

    end

    def self.start_listener(app_config)
      fork do
        listener = ::Perkins::Listener.new
        listener.app = app_config
        listener.run!
      end
    end
    ####################################

    def default_data
      { year: Time.now.year, app: @app}
    end

    helpers do
      def status_label(status)
        label = status == true ? "success" : "default"
        msg = status ? "passed" : "error"
        "<span class='label label-#{label}'>build | #{msg}</span>"
      end

      def status_class(status)
        status ? "glyphicon glyphicon-ok" : "glyphicon glyphicon-remove"
      end

      def avatar_url(email, size)
        gravatar_id = Digest::MD5.hexdigest(email.downcase)
        "http://gravatar.com/avatar/#{gravatar_id}.png?s=#{size}"
      end

      def commit_url(repo, sha)
        "#{repo.http_url}/commit/#{sha}"
      end
    end
  end

end